/*
----------------------------------------------------------------------

	WS2812B Wand

	Alain Chebrou

	WS2812B.c	Utilities for WS2812B color LEDs

	When		Who	What
	20/11/21	ac	Creation
	08/12/21	ac	V1.1	wZero() betterd end of flag
	09/12/21	ac	V1.2	Cleaning

----------------------------------------------------------------------
*/

#include	"aa.h"
#include	"aaprintf.h"
#include	"aaUtils.h"		// For aaRandom1()

#include	"string.h"
#include	"stdlib.h"		// For atoi()
#include	"ctype.h"

#include	"WS2812B.h"

//--------------------------------------------------------------------------------
//------------------------------------------------------------------
//	We want a SPI baud rate of 2.5 MHz.
//	=> CPU speed need to be 2.5 * 16 = 40 MHz, or 2.5 * 32 = 80 MHz
//	Use only MOSI pin

#define	SPI_BAUDRATE	2500000

// SPI descriptor for STM32F411
static	const spiDesc_t	wsSpiDesc =
{
	10,								// id
	RCC_APB2ENR_SPI1EN_Pos,			// SPI periph pos
	SPI1,							// Pointer to SPI registers
	{	'A',	7,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_LOW   },		// MOSI
	{	'0',	6,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_LOW   },		// MISO
	{	'0',	5,	AA_GPIO_AF_5,	AA_GPIO_MODE_ALT_PP | AA_GPIO_SPEED_LOW   },		// SCK
	{	'0',	4,	AA_GPIO_AF_0,	AA_GPIO_MODE_OUTPUT_PP | AA_GPIO_SPEED_MEDIUM },	// NSS select USER/SOFT
	{	// DMA Tx
		2,									//	DMA number 1 or 2, 0 => unused descriptor
		DMA_TO_STREAMCHAN (3, 3),			//	DMA_TO_STREAMCHAN (stream number 0..7, channel number 0..7)
		SPI_DMA_INT_PRIORITY,				//	DMA Interrupt priority
		LL_DMA_DIRECTION_MEMORY_TO_PERIPH |	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
		LL_DMA_MODE_NORMAL                |	//	Mode
		LL_DMA_PRIORITY_HIGH,				//	DMA priority
	},
	{	// DMA Rx
		0,									//	DMA number 1 or 2, 0 => unused descriptor
		DMA_TO_STREAMCHAN (2, 3),			//	DMA_TO_STREAMCHAN (stream number 0..7, channel number 0..7)
		SPI_DMA_INT_PRIORITY,				//	DMA Interrupt priority
		LL_DMA_DIRECTION_PERIPH_TO_MEMORY |	//	Direction, e.g. LL_DMA_DIRECTION_PERIPH_TO_MEMORY
		LL_DMA_MODE_NORMAL                |	//	Mode
		LL_DMA_PRIORITY_HIGH,				//	DMA priority
	},
} ;

static	spiHandle_t	spiHdl ;				// The master SPI handle returned by spiMasterConfigure()

//--------------------------------------------------------------------------------
//--------------------------------------------------------------------------------
//	Some routines for a smooth translation from a color to another one

typedef struct
{
	int16_t		rCurrent, gCurrent, bCurrent ;	// Current color component << 7
	int16_t		rInc, gInc, bInc ;				// Color component increment
	int32_t		cEnd ;							// End color
	uint32_t	steps ;		// Count of steps of the fading

} cFading_t ;

// Initialize the fading. The current color is the start color

//--------------------------------------------------------------------------------

void	wFadingInit (uint32_t cStart, uint32_t cEnd, uint32_t steps, cFading_t * pFading)
{
	int32_t	delta ;
	int32_t	cTemp ;

	pFading->rCurrent 	= (int16_t) ((cStart & 0x00FF0000u) >> 9) ;
	pFading->gCurrent 	= (int16_t) ((cStart & 0x0000FF00u) >> 1) ;
	pFading->bCurrent 	= (int16_t) ((cStart & 0x000000FFu) << 7) ;
	pFading->cEnd		= cEnd ;
	pFading->steps		= steps ;

	cTemp = ((cEnd & 0x00FF0000u) >> 9) ;
	delta = cTemp - pFading->rCurrent ;
	delta /= (int32_t) steps ;
	pFading->rInc = (int16_t) delta ;

	cTemp = ((cEnd & 0x0000FF00u) >> 1) ;
	delta = cTemp - pFading->gCurrent ;
	delta /= (int32_t) steps ;
	pFading->gInc = (int16_t) delta ;

	cTemp = ((cEnd & 0x000000FFu) << 7) ;
	delta = cTemp - pFading->bCurrent ;
	delta /= (int32_t) steps ;
	pFading->bInc = (int16_t) delta ;
}

//--------------------------------------------------------------------------------
// Compute the next color, return 0 if ended

uint32_t	wFadind (cFading_t * pFading)
{
	if (pFading->steps > 0u)
	{
		if (pFading->steps == 1u)
		{
			pFading->rCurrent 	= (int16_t) ((pFading->cEnd & 0x00FF0000u) >> 9) ;
			pFading->gCurrent 	= (int16_t) ((pFading->cEnd & 0x0000FF00u) >> 1) ;
			pFading->bCurrent 	= (int16_t) ((pFading->cEnd & 0x000000FFu) << 7) ;
		}
		else
		{
			pFading->rCurrent 	+= pFading->rInc ;
			pFading->gCurrent 	+= pFading->gInc ;
			pFading->bCurrent 	+= pFading->bInc ;
		}
		pFading->steps-- ;
		return 1 ;		// Not ended
	}
	return 0 ;			// Ended
}

//--------------------------------------------------------------------------------
// Return the current color

uint32_t	wFadindGet (cFading_t * pFading)
{
	return	((pFading->rCurrent << 9) & 0x00FF0000u) |
			((pFading->gCurrent << 1) & 0x0000FF00u) |
			((pFading->bCurrent >> 7) & 0x000000FFu) ;
}

//--------------------------------------------------------------------------------
//--------------------------------------------------------------------------------
//	The user routines
//--------------------------------------------------------------------------------
//	The LEDs configuration
//	For the test:
//		1 LED for DOUT level translation
//		+ A 8x8 square => 64 LEDs

#define		WLED_N			35	// 1 + 34	// 64 of the square + 1
#define		WSLED_BYTES		9	// Count of DMA byte per LED

// The pixel buffer
uint32_t	wLeds [WLED_N]  ;

// The DMA buffer
uint8_t		wDmaBuffer [(WLED_N + 2) * WSLED_BYTES] ;	// WLED_N + inital LED + reset time

// For the fading of 1 color
static	cFading_t		tFading ;

//--------------------------------------------------------------------------------
// Shut off all the LEDs

static	void	wZero (void)
{
	uint32_t	nByte ;

	aaMEMSET (wLeds, 0, sizeof (wLeds)) ;
	nByte = wsGenerateBuffer (wLeds, wDmaBuffer, WLED_N) ;
	wsSpiSend (spiHdl, wDmaBuffer, nByte) ;
}

//--------------------------------------------------------------------------------
//	French flag

static void	wFlagFrench (uint32_t pos, uint32_t cnt)
{
	uint32_t	nByte ;
	uint32_t	index, ii ;

	aaMEMSET (wLeds, 0, sizeof (wLeds)) ;


	if (pos + (3 * cnt) >= WLED_N)
	{
		return ;
	}
	index = pos ;
	for (ii = 0 ; ii < cnt ; ii++)
	{
		wLeds [index]				= BLUE3 ;
		wLeds [index + cnt]			= WHITE4 ;
		wLeds [index + cnt + cnt]	= RED3 ;
		index++ ;
	}
	nByte = wsGenerateBuffer (wLeds, wDmaBuffer, WLED_N) ;
	wsSpiSend (spiHdl, wDmaBuffer, nByte) ;
}

//--------------------------------------------------------------------------------
//	Complete show

//  Progressive caterpillar: 4 LEDs, use LEDs 1 to 28 so 24 jump

// The fading structs are used by many effects, not only caterpillar
static	cFading_t	wLedFading0, wLedFading1, wLedFading2, wLedFading3 ;
#define	CAT_STEPS	4
#define	CAT_LEDS	4		// Count of on LEDs
#define	CAT_JUMPS	(WLED_N - CAT_STEPS)

// Delay is the micro setp delay

static	void	wCaterUp (uint32_t colorL, uint32_t colorM, uint32_t colorH, uint32_t delay)
{
	uint32_t	iLedHead = CAT_LEDS ;
	uint32_t	nByte ;
	uint32_t	ii ;

	wLeds [iLedHead - 0] = BLACK1 ;
	wLeds [iLedHead - 1] = colorH ;
	wLeds [iLedHead - 2] = colorM ;
	wLeds [iLedHead - 3] = colorL ;
	nByte = wsGenerateBuffer (wLeds, wDmaBuffer, WLED_N) ;
	wsSpiSend (spiHdl, wDmaBuffer, nByte) ;
	aaTaskDelay (delay) ;

	while (iLedHead <= CAT_LEDS + CAT_JUMPS)
	{
		wFadingInit (BLACK1, colorH, CAT_STEPS, & wLedFading0) ;
		wFadingInit (colorH, colorM, CAT_STEPS, & wLedFading1) ;
		wFadingInit (colorM, colorL, CAT_STEPS, & wLedFading2) ;
		wFadingInit (colorL, BLACK1, CAT_STEPS, & wLedFading3) ;

		// Do micro steps
		for (ii = 0 ; ii < CAT_STEPS ; ii++)
		{
			wFadind (& wLedFading0) ;
			wFadind (& wLedFading1) ;
			wFadind (& wLedFading2) ;
			wFadind (& wLedFading3) ;
			wLeds [iLedHead - 0] =	wFadindGet (& wLedFading0) ;
			wLeds [iLedHead - 1] =	wFadindGet (& wLedFading1) ;
			wLeds [iLedHead - 2] =	wFadindGet (& wLedFading2) ;
			wLeds [iLedHead - 3] =	wFadindGet (& wLedFading3) ;
			nByte = wsGenerateBuffer (wLeds, wDmaBuffer, WLED_N) ;
			wsSpiSend (spiHdl, wDmaBuffer, nByte) ;
			aaTaskDelay (delay) ;
		}
		iLedHead ++ ;
	}
}

static	void	wCaterDown (uint32_t colorL, uint32_t colorM, uint32_t colorH, uint32_t delay)
{
	uint32_t	iLedHead = CAT_JUMPS + 1 ;	// The black LED
	uint32_t	nByte ;
	uint32_t	ii ;

	wLeds [iLedHead + 0] = BLACK1 ;
	wLeds [iLedHead + 1] = colorH ;
	wLeds [iLedHead + 2] = colorM ;
	wLeds [iLedHead + 3] = colorL ;
	nByte = wsGenerateBuffer (wLeds, wDmaBuffer, WLED_N) ;
	wsSpiSend (spiHdl, wDmaBuffer, nByte) ;
	aaTaskDelay (delay) ;

	while (iLedHead > 0u)
	{
		wFadingInit (BLACK1, colorH, CAT_STEPS, & wLedFading0) ;
		wFadingInit (colorH, colorM, CAT_STEPS, & wLedFading1) ;
		wFadingInit (colorM, colorL, CAT_STEPS, & wLedFading2) ;
		wFadingInit (colorL, BLACK1, CAT_STEPS, & wLedFading3) ;

		// Do micro steps
		for (ii = 0 ; ii < CAT_STEPS ; ii++)
		{
			wFadind (& wLedFading0) ;
			wFadind (& wLedFading1) ;
			wFadind (& wLedFading2) ;
			wFadind (& wLedFading3) ;
			wLeds [iLedHead + 0] =	wFadindGet (& wLedFading0) ;
			wLeds [iLedHead + 1] =	wFadindGet (& wLedFading1) ;
			wLeds [iLedHead + 2] =	wFadindGet (& wLedFading2) ;
			wLeds [iLedHead + 3] =	wFadindGet (& wLedFading3) ;
			nByte = wsGenerateBuffer (wLeds, wDmaBuffer, WLED_N) ;
			wsSpiSend (spiHdl, wDmaBuffer, nByte) ;
			aaTaskDelay (delay) ;
		}
		iLedHead -- ;
	}
}

//--------------------------------------------------------------------------------
// Final fading from 2nd to 3rd color then fading to black

static	void	wHeadFinal (uint32_t colorStart, uint32_t colorEnd, uint32_t delay)
{
	uint32_t	nByte ;
	uint32_t	ii ;
	uint32_t	color ;

	wLeds [WLED_N-4] = colorStart ;
	wLeds [WLED_N-3] = colorStart ;
	wLeds [WLED_N-2] = colorStart ;
	wLeds [WLED_N-1] = colorStart ;
	nByte = wsGenerateBuffer (wLeds, wDmaBuffer, WLED_N) ;
	wsSpiSend (spiHdl, wDmaBuffer, nByte) ;
	aaTaskDelay (delay*4) ;

	// Fading from colorStart to colorEnd
	wFadingInit (colorStart, colorEnd, 10, & tFading) ;
	for (ii = 0u ; ii < 20 ; ii++)
	{
		wFadind (& tFading) ;
		color =	wFadindGet (& tFading) ;
		wLeds [WLED_N-4] = color ;
		wLeds [WLED_N-3] = color ;
		wLeds [WLED_N-2] = color ;
		wLeds [WLED_N-1] = color ;
		nByte = wsGenerateBuffer (wLeds, wDmaBuffer, WLED_N) ;
		wsSpiSend (spiHdl, wDmaBuffer, nByte) ;
		aaTaskDelay (delay) ;
	}

	// Fading to black
	wFadingInit (colorEnd, BLACK1, 10, & tFading) ;
	for (ii = 0u ; ii < 10 ; ii++)
	{
		wFadind (& tFading) ;
		color =	wFadindGet (& tFading) ;
		wLeds [WLED_N-4] = color ;
		wLeds [WLED_N-3] = color ;
		wLeds [WLED_N-2] = color ;
		wLeds [WLED_N-1] = color ;
		nByte = wsGenerateBuffer (wLeds, wDmaBuffer, WLED_N) ;
		wsSpiSend (spiHdl, wDmaBuffer, nByte) ;
		aaTaskDelay (delay) ;
	}
}
//--------------------------------------------------------------------------------
//	Blink the head

static	void	wHeadBlink (uint32_t color, uint32_t DelayOn, uint32_t DelayOff, uint32_t count)
{
	uint32_t	nByte ;
	uint32_t	ii ;

	for (ii = 0u ; ii < count ; ii++)
	{
		wLeds [WLED_N-4] = color ;
		wLeds [WLED_N-3] = color ;
		wLeds [WLED_N-2] = color ;
		wLeds [WLED_N-1]   = color ;
		nByte = wsGenerateBuffer (wLeds, wDmaBuffer, WLED_N) ;
		wsSpiSend (spiHdl, wDmaBuffer, nByte) ;
		aaTaskDelay (DelayOn) ;

		wLeds [WLED_N-4] = BLACK1 ;
		wLeds [WLED_N-3] = BLACK1 ;
		wLeds [WLED_N-2] = BLACK1 ;
		wLeds [WLED_N-1]   = BLACK1 ;
		nByte = wsGenerateBuffer (wLeds, wDmaBuffer, WLED_N) ;
		wsSpiSend (spiHdl, wDmaBuffer, nByte) ;
		aaTaskDelay (DelayOff) ;
	}
}

//--------------------------------------------------------------------------------

#define	CAT_CNT			8		// Count of pattern in catList
#define	CAT_DELAY		4		// Delay between move

static	const	uint32_t	catList [CAT_CNT * 3] =
{
	IRED,		IWHITE,		IORANGE,
	IGREEN,		IYELLOW,	ICYAN,
	IBLUE,		ICYAN,		IGREEN,
	IORANGE,	IWHITE,		IRED,
	IYELLOW,	IGREEN,		IORANGE,
	IMAGENTA,	IRED,		IYELLOW,
	ICYAN,		IGREEN,		IYELLOW,
	IWHITE,		IRED,		IYELLOW,
} ;

// Caterpillar up followed by head effect
static	void	wCaterpillar (uint32_t ix, uint32_t bBlink)
{
	if (ix >= CAT_CNT)
	{
		ix = 0u ;
	}
	ix *= 3u ;

	wCaterUp (wPalette [catList[ix]+4], wPalette [catList[ix]+2], wPalette [catList[ix]], CAT_DELAY) ;

	// Final
	if (bBlink != 0u)
	{
		// Blinking final with 2nd color
		wHeadBlink (wPalette [catList[ix+1]], 200, 100, 5) ;
	}
	else
	{
		// Fading from 2nd to 3rd color then fading to black
		wHeadFinal (wPalette [catList[ix+1]], wPalette [catList[ix+2]], 50) ;
	}
}

// Caterpillar up followed by caterpillar down
static	void	wCaterUpDown (uint32_t ix)
{
	// Chenille
	if (ix >= CAT_CNT)
	{
		ix = 0u ;
	}
	ix *= 3u ;

	wCaterUp (wPalette [catList[ix]+4], wPalette [catList[ix]+2], wPalette [catList[ix]], CAT_DELAY) ;

	wCaterDown (wPalette [catList[ix]+4], wPalette [catList[ix]+2], wPalette [catList[ix]], CAT_DELAY) ;
}

//--------------------------------------------------------------------------------
//	Sparkle some LEDs

#define	SPARK_STEPS		10u		// Fading steps
#define	SPARK_DELAY		5u		// Delay between steps
#define	SPARK_COUNT		30u		// Count of sparkle
#define	SPARK_LOOP		(SPARK_STEPS * SPARK_COUNT)

#define	SPARK_CNT		6		// Count of color pair

// Sparkle 2 LEDs, one of each color in a sparkList array line
// Use LEDs 1 to WLED_N-1 (doesn't use LED 0)

static const uint32_t sparkList [SPARK_CNT*2] =
{
//	LED 0		LED 1
	IRED,		IORANGE,
	IRED,		IYELLOW,
	IGREEN,		ICYAN,
	IBLUE,		IMAGENTA,
	IWHITE,		ICYAN,
	IWHITE,		IYELLOW
} ;

static	void	wSparkle (uint32_t ix)
{
	uint32_t	pos0, pos1 ;
	uint32_t	color0, color1 ;
	uint32_t	nByte ;
	uint32_t	ii ;

	// Start the 1st LED
	color0 = wPalette [sparkList [ix * 2]] ;
	pos0 = (aaRandom1 () % (WLED_N-1)) + 1u ;
	wFadingInit (color0, BLACK1, SPARK_STEPS, & wLedFading0) ;
	wLeds [pos0] =	color0;

	// Add a delay to start the 2nd LED (the delay is 1/2 fading duration)
	color1 = wPalette [sparkList [ix * 2 + 1]] ;
	pos1 = (aaRandom1 () % (WLED_N-1)) + 1u ;
	wFadingInit (BLACK1, BLACK1, SPARK_STEPS/2u, & wLedFading1) ;
	wLeds [pos1] =	BLACK1 ;

	nByte = wsGenerateBuffer (wLeds, wDmaBuffer, WLED_N) ;
	wsSpiSend (spiHdl, wDmaBuffer, nByte) ;

	for (ii = 0u ; ii < SPARK_LOOP ; ii++)
	{
		if (0u == wFadind (& wLedFading0))
		{
			// start new fading of LED 0
			do
			{
				pos0 = (aaRandom1 () % (WLED_N-1)) + 1u ;
			} while (pos0 == pos1) ;
			wFadingInit (color0, BLACK1, SPARK_STEPS, & wLedFading0) ;
		}
		if (0u == wFadind (& wLedFading1))
		{
			// start new fading of LED 1
			do
			{
				pos1 = (aaRandom1 () % (WLED_N-1)) + 1u ;
			} while (pos1 == pos0) ;
			wFadingInit (color1, BLACK1, SPARK_STEPS, & wLedFading1) ;
		}
if (pos0 == 0u  ||  pos1 == 0u  || wLeds [0] != 0u)
{
	aaPrintf ("p0=%u  p1=%u c0=%08X\n", pos0, pos1, wLeds [0]) ;
}
		wLeds [pos0] = wFadindGet (& wLedFading0) ;
		wLeds [pos1] = wFadindGet (& wLedFading1) ;
		nByte = wsGenerateBuffer (wLeds, wDmaBuffer, WLED_N) ;
		wsSpiSend (spiHdl, wDmaBuffer, nByte) ;
		aaTaskDelay (SPARK_DELAY) ;
	}
	wZero () ;
}

//--------------------------------------------------------------------------------
//	The main entry point of the WS2812B routines

//	bAuto=1	Automatic mode: start an animation on every button press
//	bAuto=0	Manual mode: wait for a char between 0 and 7
//	In both mode the char 'q' allows to quit the wand() function

void	wand (uint32_t	bAuto)
{
	uint32_t	rnd ;

	aaPrintf ("Wand WS2812B V1.2\n'q' for quit\n") ;

	wsSpiConfigure (& wsSpiDesc, SPI_BAUDRATE, & spiHdl) ;

	while (1)
	{
		if (bAuto == 0u)
		{
			// Manual mode
			rnd = aaGetChar () ;
			if (rnd == 'q')
			{
				return ;
			}
			wZero () ;

			if (isdigit (rnd) != 0)
			{
				rnd = rnd - 0x30 ;
				if (rnd > 7u)
				{
					rnd = 0u ;
				}
			}
			else
			{
				rnd = (aaRandom1 () >> 8) % 8u ;
			}
		}
		else
		{
			// Automatic mode
			if (aaCheckGetChar () != 0u)
			{
				rnd = aaGetChar () ;
				if (rnd == 'q')
				{
					return ;	// Go to manual mode
				}
			}

			if (bspInput (BSP_BUTTON0) == 1u)
			{
				aaTaskDelay (50) ;	// Button non pressed
				continue ;
			}

			rnd = (aaRandom1 () >> 8) % 8u ;
			wZero () ;		// To clean the flag
		}

		switch (rnd)
		{
			case 0:
			case 1:
				rnd = ((aaRandom1 () % 10u) > 6u) ? 1 : 0 ;		// 3/10 blink
				wCaterpillar (aaRandom1 () % CAT_CNT, rnd) ;
				break ;

			case 7:
			case 5:
				rnd = aaRandom1 () % CAT_CNT ;
				wCaterUpDown (rnd) ;
				wZero () ;
				break ;

			case 2:
			case 3:
				rnd = aaRandom1 () % SPARK_CNT ;
				wSparkle (rnd) ;
				break ;

			case 6:
				wFlagFrench (WLED_N - 9 - 1, 3) ;
				while (bspInput (BSP_BUTTON0) == 0u)
				{
					aaTaskDelay (50) ;	// Button pressed
				}
				break ;

			case 4:
				wCaterUp (WHITE5, WHITE3, RED1, CAT_DELAY) ;
				wCaterDown (WHITE5, WHITE3, RED1, CAT_DELAY) ;
				wZero () ;
				break ;

			default:
				break ;
		}
	}
}

// A function to test some elementary functions and set 1 LED  color
void	wsColor (void)
{
	uint32_t	color ;
	char		str [20] ;
	char		* pStr, * pMemo ;
	uint32_t	nLed, nByte ;

	wsSpiConfigure (& wsSpiDesc, SPI_BAUDRATE, & spiHdl) ;

	aaPrintf ("N R G B: ") ;
	aaGets (str, sizeof (str)) ;

	pStr = strtok_r (str, " .", & pMemo) ;
	if (* pStr == 'z')
	{
		// Shut off all the LEDs
		wZero () ;
		return ;
	}
	if (* pStr == 'w')
	{
		// Wand in manual mode
		wand (0u) ;
		return ;
	}
	if (* pStr == 's')
	{
		// Test sparkle
		wSparkle (0) ;
		return ;
	}
	if (* pStr == 'f')
	{
		// Display French flag
		wFlagFrench (9, 2) ;
		return ;
	}
	if (* pStr == 'p')
	{
		// Display the color palette
		// WARNING: Need > 65 LEDs DMA buffer
		nByte = wsGenerateBuffer (wPalette, wDmaBuffer,
								 (WLED_N >= wPaletteCount) ? wPaletteCount : WLED_N) ;
		wsSpiSend (spiHdl, wDmaBuffer, nByte) ;
		return ;
	}
	if (* pStr == 'c')
	{
		// Test caterpillar with a color
		color = 0 ;
		if (isdigit (pStr[1]))
		{
			color = atoi (pStr+1) ;
		}
		wCaterpillar (color, 1u) ;
		return ;
	}

	// Display the requested color on the requested LED
	// the command is: c rrr.ggg.bbb
	nLed = atoi (pStr) ;
	if (nLed >= WLED_N)
	{
		aaPrintf ("Only %u LEDs\n", WLED_N) ;
		return ;
	}

	color = 0u ;

	pStr = strtok_r (NULL, " .", & pMemo) ;
	nByte = atoi (pStr) ;
	color |= (nByte << 16) ;			// Red

	pStr = strtok_r (NULL, " .", & pMemo) ;
	nByte = atoi (pStr) ;
	color |= (nByte << 8) ;				// Green

	pStr = strtok_r (NULL, " .", & pMemo) ;
	nByte = atoi (pStr) ;
	color |= nByte ;					// Blue

//	wLeds [nLed] = color ;
//	nByte = wsGenerateBuffer (wLeds, wDmaBuffer, WLED_N) ;
wLeds [0] = color ;
nByte = wsGenerateBuffer (wLeds, wDmaBuffer, 2) ;
	wsSpiSend (spiHdl, wDmaBuffer, nByte) ;
}

//--------------------------------------------------------------------------------
// The first simple test:

#define		LED_NB		4

// The colors buffers of the LEDs
uint32_t	motif_1   [LED_NB] =	{ 0x00800000, 0x00008000, 0x00000080, 0x00808000 } ;
uint32_t	motif_2   [LED_NB] =	{ 0x00008000, 0x00000080, 0x00808000, 0x00800000 } ;
uint32_t	motif_3   [LED_NB] =	{ 0x00000080, 0x00808000, 0x00800000, 0x00008000 } ;
uint32_t	motif_Off [LED_NB] =	{ 0x00000000, 0x00000000, 0x00000000, 0x00000000 } ;

uint8_t		dmaBuffer [LED_NB * WSLED_BYTES] ;

void	wsTest (void)
{
	uint32_t	nByte ;

	wsSpiConfigure (& wsSpiDesc, SPI_BAUDRATE, & spiHdl) ;

	// Build buffer
	nByte = wsGenerateBuffer (motif_1, dmaBuffer, LED_NB) ;

	// Transmit
	wsSpiSend (spiHdl, dmaBuffer, nByte) ;

	(void) aaGetChar () ;
	nByte = wsGenerateBuffer (motif_2, dmaBuffer, LED_NB) ;
	wsSpiSend (spiHdl, dmaBuffer, nByte) ;

	(void) aaGetChar () ;
	nByte = wsGenerateBuffer (motif_3, dmaBuffer, LED_NB) ;
	wsSpiSend (spiHdl, dmaBuffer, nByte) ;

	// Turn off the LED
	(void) aaGetChar () ;
	nByte = wsGenerateBuffer (motif_Off, dmaBuffer, LED_NB) ;
	wsSpiSend (spiHdl, dmaBuffer, nByte) ;
}

//--------------------------------------------------------------------------------
